@@ -21,6 +21,40 @@ except AttributeError:  # pragma: no cover - older Pillow fallback
     _LANCZOS = Image.LANCZOS
 
 _DEFAULT_SIZE = (1024, 1024)
+
+
+class ProviderExecutionError(RuntimeError):
+    """Raised when an upstream provider cannot complete an image request."""
+
+    def __init__(
+        self,
+        *,
+        provider: str,
+        generator: str,
+        message: str,
+        status: Optional[int] = None,
+        body: Optional[str] = None,
+        context: Optional[Dict[str, object]] = None,
+    ) -> None:
+        super().__init__(message)
+        self.provider = provider
+        self.generator = generator
+        self.status = status
+        self.body = body
+        self.context: Dict[str, object] = context.copy() if context else {}
+
+    def attach_context(self, extra: Dict[str, object]) -> None:
+        for key, value in extra.items():
+            self.context.setdefault(key, value)
+
+
+def _truncate(text: str, limit: int = 2000) -> str:
+    snippet = text.strip()
+    if len(snippet) <= limit:
+        return snippet
+    return snippet[: limit - 1] + "…"
+
+
 class ImageGenerator(Protocol):
     def generate(
         self,
@@ -192,6 +226,14 @@ class OpenRouterGenerator:
             target_ratio = target_size[0] / target_size[1]
 
         model_id = model or os.environ.get("IMAGEMCP_OPENROUTER_MODEL") or "google/gemini-2.5-flash-image-preview"
+        provider_name = provider or provider_key
+        request_context = {
+            "model": model_id,
+            "requested_total": total,
+            "size": size,
+            "aspect_ratio": aspect_ratio,
+        }
+
         try:
             while len(images) < total:
                 remaining = total - len(images)
@@ -220,20 +262,34 @@ class OpenRouterGenerator:
                         break
                 if not new_images:
                     break
+
+            if not images:
+                debug_message = ""
+                if last_response is not None:
+                    try:
+                        debug_message = json.dumps(last_response, indent=2)
+                    except Exception:
+                        debug_message = str(last_response)
+                raise ProviderExecutionError(
+                    provider=provider_name,
+                    generator="openrouter",
+                    message="OpenRouter did not return any image files.",
+                    body=_truncate(debug_message) if debug_message else None,
+                    context={**request_context, "reason": "empty_response"},
+                )
+        except ProviderExecutionError as exc:
+            exc.attach_context(request_context)
+            raise
+        except Exception as exc:
+            raise ProviderExecutionError(
+                provider=provider_name,
+                generator="openrouter",
+                message=str(exc),
+                context={**request_context, "reason": "unexpected_exception"},
+            ) from exc
         finally:
             self.client.close()
 
-        if not images:
-            debug_message = ""
-            if last_response is not None:
-                try:
-                    debug_message = json.dumps(last_response, indent=2)
-                except Exception:
-                    debug_message = str(last_response)
-            raise RuntimeError(
-                f"OpenRouter did not return any image files. {debug_message}" if debug_message else "OpenRouter did not return any image files."
-            )
-
         artifacts: List[GeneratedImageArtifacts] = []
         for index, entry in enumerate(images[:total]):
             data_url = entry["data_url"]
@@ -373,17 +429,50 @@ class OpenRouterGenerator:
         try:
             response = self.client.post(self.api_url, json=payload)
         except httpx.RequestError as exc:  # pragma: no cover - network dependent
-            raise RuntimeError(f"Failed to reach OpenRouter API: {exc}") from exc
-        try:
-            response.raise_for_status()
-        except httpx.HTTPStatusError as exc:
-            raise RuntimeError(
-                f"OpenRouter API error {exc.response.status_code}: {exc.response.text}"
+            raise ProviderExecutionError(
+                provider="openrouter",
+                generator="openrouter",
+                message="Failed to reach OpenRouter API.",
+                context={
+                    "reason": "network",
+                    "url": self.api_url,
+                    "error": str(exc),
+                },
             ) from exc
+
+        status = response.status_code
+        if status >= 400:
+            body = _truncate(response.text or "")
+            context = {
+                "reason": "http_status",
+                "url": self.api_url,
+            }
+            request_id = response.headers.get("x-request-id")
+            if request_id:
+                context["request_id"] = request_id
+            retry_after = response.headers.get("retry-after")
+            if retry_after:
+                context["retry_after"] = retry_after
+            raise ProviderExecutionError(
+                provider="openrouter",
+                generator="openrouter",
+                message=f"OpenRouter API returned HTTP {status}",
+                status=status,
+                body=body,
+                context=context,
+            )
+
         try:
             return response.json()
         except ValueError as exc:
-            raise RuntimeError("Failed to decode OpenRouter response as JSON.") from exc
+            raise ProviderExecutionError(
+                provider="openrouter",
+                generator="openrouter",
+                message="OpenRouter response was not valid JSON.",
+                status=status,
+                body=_truncate(response.text or ""),
+                context={"reason": "invalid_json", "url": self.api_url},
+            ) from exc
 
 
 def build_generator(output_dir: Path, kind: Optional[str] = None) -> ImageGenerator:
@@ -637,6 +726,7 @@ def _stable_random(seed: Optional[int]) -> random.Random:
 
 
 __all__ = [
+    "ProviderExecutionError",
     "ImageGenerator",
     "MockImageGenerator",
     "AiSdkGenerator",
