@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import json
 import re
+import shutil
 from pathlib import Path
 from typing import Dict, Iterable, Iterator, List, Optional, Tuple
 
@@ -21,6 +22,8 @@ class ProjectPaths:
     project_root: Path
     target_root: Path
     sessions_root: Path
+    campaigns_root: Path
+    templates_root: Path
 
     @classmethod
     def create(cls, project_root: Path, target_root: Path) -> "ProjectPaths":
@@ -30,15 +33,27 @@ class ProjectPaths:
             (project_root / CONFIG_DIR_NAME / ".sessions").resolve(),
             project_root,
         )
+        campaigns_root = _ensure_within_root(
+            (project_root / CONFIG_DIR_NAME / "campaigns").resolve(),
+            project_root,
+        )
+        templates_root = _ensure_within_root(
+            (project_root / CONFIG_DIR_NAME / "templates").resolve(),
+            project_root,
+        )
         return cls(
             project_root=project_root,
             target_root=target_root,
             sessions_root=sessions_root,
+            campaigns_root=campaigns_root,
+            templates_root=templates_root,
         )
 
     def ensure_directories(self) -> None:
         self.target_root.mkdir(parents=True, exist_ok=True)
         self.sessions_root.mkdir(parents=True, exist_ok=True)
+        self.campaigns_root.mkdir(parents=True, exist_ok=True)
+        self.templates_root.mkdir(parents=True, exist_ok=True)
 
     def target_for_slot(self, slot: str, extension: str = ".png") -> Path:
         return self.target_root / f"{slot}{extension}"
@@ -63,6 +78,24 @@ class ProjectPaths:
     def manifest_path(self, session_dir: Path) -> Path:
         return session_dir / "session.json"
 
+    def campaign_dir(self, campaign_id: str) -> Path:
+        return _ensure_within_root((self.campaigns_root / campaign_id).resolve(), self.project_root)
+
+    def campaign_routes_dir(self, campaign_id: str) -> Path:
+        return self.campaign_dir(campaign_id) / "routes"
+
+    def campaign_placements_dir(self, campaign_id: str) -> Path:
+        return self.campaign_dir(campaign_id) / "placements"
+
+    def campaign_thumbnails_dir(self, campaign_id: str) -> Path:
+        return self.campaign_dir(campaign_id) / "thumbnails"
+
+    def campaign_exports_dir(self, campaign_id: str) -> Path:
+        return self.campaign_dir(campaign_id) / "exports"
+
+    def campaign_logs_dir(self, campaign_id: str) -> Path:
+        return self.campaign_dir(campaign_id) / "logs"
+
 
 def _validate_slug(value: str) -> str:
     if not re.fullmatch(r"[a-z0-9_-]+", value):
@@ -91,6 +124,13 @@ def load_manifest(manifest_path: Path) -> SessionManifest:
     return SessionManifest.from_dict(data)
 
 
+@dataclass()
+class SlotDeletionResult:
+    slot: str
+    removed_sessions: int
+    removed_targets: int
+
+
 def list_manifests_for_slot(paths: ProjectPaths, slot: str) -> List[SessionManifest]:
     manifests: List[SessionManifest] = []
     safe_slot = _validate_slug(slot)
@@ -125,12 +165,48 @@ def most_recent_session(manifests: Iterable[SessionManifest]) -> Optional[Sessio
     return latest
 
 
+def delete_slot(paths: ProjectPaths, slot: str) -> SlotDeletionResult:
+    safe_slot = _validate_slug(slot)
+    manifests = list_manifests_for_slot(paths, safe_slot)
+    target_paths = set()
+    for manifest in manifests:
+        for candidate in (manifest.selected_path, manifest.target_path):
+            if not candidate:
+                continue
+            target = (paths.project_root / candidate).resolve()
+            try:
+                target.relative_to(paths.project_root)
+            except ValueError:
+                continue
+            target_paths.add(target)
+    # Always include the default target path in case no manifest exists.
+    default_target = paths.target_for_slot(safe_slot)
+    target_paths.add(default_target)
+
+    removed_targets = 0
+    for path in target_paths:
+        if path.exists() and path.is_file():
+            path.unlink()
+            removed_targets += 1
+
+    session_dirs = list(paths.sessions_root.glob(f"{safe_slot}_*"))
+    removed_sessions = 0
+    for session_dir in session_dirs:
+        if session_dir.is_dir():
+            shutil.rmtree(session_dir)
+            removed_sessions += 1
+
+    return SlotDeletionResult(slot=safe_slot, removed_sessions=removed_sessions, removed_targets=removed_targets)
+
+
 __all__ = [
     "ProjectPaths",
     "InvalidPathError",
     "write_manifest",
     "load_manifest",
+    "SlotDeletionResult",
     "list_manifests_for_slot",
     "list_all_slots",
     "most_recent_session",
+    "delete_slot",
 ]
